/*
 * InCode - [ input.c ]
 * (c) 2002, J. Weeks
 * jweeks@subdimension.com
 *
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdio.h>
#include <ctype.h>

#include "input.h"
#include "iswhat.h"

FILE *input;

/**
 * A "Look Ahead" character.
 * Holds the value of the last character read, and used as a basis
 * of compiling branch prediction.
 */
char lookAhead = ' ';

/**
 * Initialize the input system.
 * Opens the input file.
 * @param fname the filename to load (/dev/stdin by default)
 * @return 0 on success
 */
int inputInit(char *fname) {
  input = fopen(fname, "r");
  return 0;
}

/**
 * Close the input system.
 * Closes the input file.
 * @return 0 on success
 */
int inputClose(void) {
  fclose(input);
  return 0;
}

/**
 * Check if input exists.
 * @return true if input exists, false if at end of input file
 */
int inputExists(void) {
  return !feof(input);
}

/**
 * Read a character into the look ahead.
 * Only the look ahead character is affected by this read.
 * @return Don't know... :)  The character read, I think
 */
int inputLookAhead(void) {
  return (lookAhead = fgetc(input));
}


/**
 * Read a character.
 * The look ahead character is also affected by this read.
 * @param c Pointer to where to store character read
 * @return meh!?
 */
int inputChar(char *c) {
  return (*c = lookAhead = fgetc(input));
}

/**
 * Read an identifier.
 * Reads an identifier into a string (terminated by any non-alphanumeric
 * character).
 * @param str The string to store identifier in.
 * @return 0 on success
 */
int inputIdent(char *str) {
  int i = 0;

  while(isAlphaNum(lookAhead)) {
    str[i++] = lookAhead;
    inputLookAhead();
  }

  str[i] = '\0';
  return 0;
}

// this will change later to support 0xffff, and 16_ffff
/**
 * Input a number.
 * Inputs a number in the base_number notation (ie, 16_ffff), also
 * supports the C/C++ 0xffff for hex numbers.
 * @param str The string to store the number in
 * @return 0 on success
 */
int inputNumber(char *str) {
  int i = 0;

  while(isNum(lookAhead)) {
    str[i++] = lookAhead;
    inputLookAhead();
  }

  str[i] = '\0';
  return 0;
}

/**
 * Reads a symbol.
 * Reads any sized symbol.
 * @param str Where to put symbol
 * @return 0 on success
 */
int inputSymbol(char *str) {
  int i = 0;

  /* This old technique looked really nice, but, unfortunately, it
   * treated two tokens together, as one.  eg. -( and ); in x = -(3/2);
  while(isSymbol(lookAhead)) {
    str[i++] = lookAhead;
    inputLookAhead();
  }
  */
  str[i++] = lookAhead;
  inputLookAhead();

  // as it stands now, the only multi-character symbols in inCode
  // have = as the second character (++, -- might be a little bit
  // more complex)
  // possible: where "str[0] == lookAhead" accounts for ++,--,&&,||,etc
  if(lookAhead == '=') {
    str[i++] = lookAhead;
    inputLookAhead();
  }

  str[i] = '\0';
  return 0;
}

/**
 * Reads an enclosed quote
 * Reads a string, disregarding outside quotes.
 * @param str Where to store the string
 * @param ch The outside quoting character
 * @return 0 on success
 */
int inputUntilChar(char *str, char ch) {
  int i = 0;

  // skip the first quote character (either ' or " )
  inputLookAhead();

  do {
    str[i++] = lookAhead;
    inputLookAhead();
  } while(lookAhead != ch);

  // skip the last quote character
  inputLookAhead();

  // add a null terminator to the new string
  str[i] = '\0';
  return 0;
}

int toNumber(char *str) {
  int number = 0, digit;
  int base = 10, place = 1, i;

  for(i = strlen(str) - 1; i >= 0; i--) {
    // the following will convert a single bin,oct,dec,hex character to a digit
    if(str[i] > '9') digit = (toupper(str[i]) - 'A') + 10;
    else digit = str[i] - '0';

    number += digit * place;
    place *= base;
  }

  return number;
}