/*
 * InCode - [ token.h ]
 * (c) 2002, J. Weeks
 * jweeks@subdimension.com
 *
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef __TOKEN_H__
#define __TOKEN_H__

/**
 * Maximum number of tokens.
 * The token engine cannot evaluate above this many tokens/line (unless
 * a linked list system is used).
 */
#define MAX_TOKEN	100

/**
 * A Token.
 * Every source code element in InCode.
 */
typedef struct {
  //struct Token *next;		/**< A link to the next token on the line */
  char name[32];				/**< An identifier's name, max 32 bytes */
  long type;						/**< The type of data, identifier or literal */
  long value;						/**< The value of a numeric literial */
  double fvalue;				/**< The value of a floating point literal */
  long size;						/**< The size of a variably sized token */
} Token;

/**
 * Token enum.
 * Contains values for all the possible tokens from the InCode syntax.
 */
enum Tokens {
  TIdentifier,			/**< An Identifier, could be variable, function or object */
  TStringLiteral,		/**< A string literal (encased in double quotes) */
  TCharacterLiteral,/**< A character literal (encased in single quotes) */
  TNumericLiteral,	/**< A numeric literal (of any base, specified in 'size') */
  TFloatLiteral,		/**< A floating point literal (of any size, specified in 'size') */
  TDataType,				/**< A datatype indentifier (char, word, dword, etc) used this, or the below? */
  TByte,
  TWord,
  TDWord,
  TQWord,
  TSingle,
  TDouble,
  TEOL,							/**< End of line character ; */
  TEnd,							/**< End of construct character - also ; */
  TLeftBracket,			/**< formatting character ( */
  TRightBracket,		/**< formatting character ) */
  TLeftSqBracket,   /**< address of character [ */
  TRightSqBracket,  /**< address of character ] */
  TLeftBrace,				/**< formatting character { */
  TRightBrace,			/**< formatting character } */
  TEqual,
  TCompEqual,				/**< == or IS operator */
  TAdd,							/**< operator + */
  TSub,							/**< operator - */
  TDiv,							/**< operator / */
  TMul,							/**< operator * */
  TExp,							/**< exponent operator ^ */
  TAddEqual,				/**< operator += */
  TSubEqual,				/**< operator -= */
  TDivEqual,				/**< operator /= */
  TMulEqual,			  /**< operator *= */
  TClass,						/**< the word CLASS */
  TIf,
  TElse,
  TDo,
  TWhile,
  TUntil,
  TLoop,
  TFor,
  TBreak,
  // All Tokens past this point are used in the compiling pass,
  // not the tokenizing or parsing pass.  TCompileTokens is simply
  // a flag, to differentiate the two types.
  TCompileTokens,
  TNegate,
  TJE,							/**< Jump if equal */
  TJNE,							/**< Jump if not equal */
  TJMP,							/**< Unconditional jump */
  TLabel,					  /**< A label to jump to */
  TEndList					/**< nothing, used internally to mark the end of this list */
};

typedef struct {
  int type;
  char *looksLike;
} TokenLookupItem;

int tokenInit(void);
int tokenClose(void);
int tokenClear(void);
int tokenize(void);
Token *tokenGet(void);
Token *tokenNew(void);
Token *tokenNewType(long type);
int tokenDelete(Token *delTok);
int tokenAdd(Token *add);
int tokenPrint(Token *tok);
int tokenPrintAll(void);
int tokenTypeFromString(char *str);

#endif